<?php

namespace Drupal\dpxu\Form;

use Drupal\dpxu\ProxyUserTools;
use Drupal\user\Entity\User;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Provides a Managed User Create User form.
 */
class CreateManagedUserForm extends FormBase {

  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * Proxy User Tools service.
   *
   * @var \Drupal\dpxu\ProxyUserTools
   */
  private $userTools;

  /**
   * Constructs the CreateManagedUserForm object.
   *
   * @param \Drupal\dpxu\ProxyUserTools $user_tools
   *   The user tools service.
   */
  public function __construct(ProxyUserTools $user_tools) {
    $this->userTools = $user_tools;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('dpxu.user_tools')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dpxu_create_user_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $current_uid = $this->currentUser()->id();
    $userTools = $this->userTools;
    $checkCreate = $userTools->getCreateManagedUserStatusAndReason($current_uid);

    if (!$checkCreate['status']) {
      $this->messenger()->addMessage($checkCreate['message'], 'error');
      $url = Url::fromRoute('entity.user.canonical', ['user' => $current_uid]);
      return new RedirectResponse($url->toString());
    }

    $form['field_dpxu_fullname'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Full name'),
      '#description' => $this->t('Please provide the full name of the person you are creating the account on behalf of, as they would wish to be addressed in communications.'),
      '#required' => TRUE,
    ];
    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#description' => $this->t('Spaces are allowed; punctuation is not allowed except for periods, hyphens, apostrophes, and underscores.'),
      '#required' => TRUE,
    ];

    $form['mail'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Email address'),
      '#description' => $this->t('A unique identifier for this account, which must validate as an email address format but is not required to be a deliverable email address. Leave blank if the user does not have a unique email address.'),
    ];

    $form['pass'] = [
      '#type' => 'password',
      '#title' => $this->t('Password'),
      '#required' => TRUE,
    ];
    $form['pass_confirm'] = [
      '#type' => 'password',
      '#description' => $this->t('Provide a password for the new account in both fields.'),
      '#required' => TRUE,
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create managed user account'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $username = $form_state->getValue('name');
    if (!empty($username)) {
      $temp_user = User::create();
      $temp_user->setUsername($username);
      $violations = $temp_user->name->validate();
      if (is_object($violations) && $violations->count() > 0) {
        $form_state->setErrorByName('name', $violations[0]->getMessage());
      }
    }
    else {
      $form_state->setErrorByName('name', $this->t('Username cannot be empty'));
    }

    if (!empty($form_state->getValue('mail'))) {
      $mail_string = $form_state->getValue('mail');
      $temp_user = User::create();
      $temp_user->setEmail($mail_string);
      $violations = $temp_user->mail->validate();
      if (is_object($violations) && $violations->count() > 0) {
        $form_state->setErrorByName('mail', $violations[0]->getMessage());
      }
    }

    $pass = $form_state->getValue('pass');
    $pass_confirm = $form_state->getValue('pass_confirm');
    if ($pass !== $pass_confirm) {
      $form_state->setErrorByName('pass');
      $form_state->setErrorByName('pass_confirm', $this->t('The specified passwords do not match.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Grab values and create a new user.
    $manager_uid = $this->currentUser()->id();

    $form_values = $form_state->getValues();

    $defined_values = [
      'mail' => $form_values['mail'] ?? '',
      'field_dpxu_fullname' => $form_values['field_dpxu_fullname'] ?? '',
      'field_dpxu_manager_uid' => $manager_uid,
      'status' => 1,
    ];

    $account_values = array_merge($defined_values, $form_values);
    $account = User::create($account_values);
    $account->addRole('dpxu_managed');
    $account->save();

    // Redirect to account list.
    $form_state->setRedirect('view.dpxu_managed_users_views.page_1', ['user' => $manager_uid]);
  }

}
