<?php

namespace Drupal\dpxu;

use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\Core\TempStore\SharedTempStoreFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Proxy User Tools service.
 */
class ProxyUserTools {

  use StringTranslationTrait;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The entity form builder service.
   *
   * @var \Drupal\Core\Entity\EntityFormBuilderInterface
   */
  protected $entityFormBuilder;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The shared tempstore service.
   *
   * @var \Drupal\Core\TempStore\SharedTempStoreFactory
   */
  protected $sharedTempStore;

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a ProxyUserTools object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entity_form_builder
   *   The entity form builder service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\Core\TempStore\SharedTempStoreFactory $shared_tempstore
   *   The shared tempstore service.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager interface.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager interface.
   */
  public function __construct(AccountProxyInterface $current_user, EntityFormBuilderInterface $entity_form_builder, EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory, SharedTempStoreFactory $shared_tempstore, MailManagerInterface $mail_manager, LanguageManagerInterface $language_manager) {
    $this->currentUser = $current_user;
    $this->entityFormBuilder = $entity_form_builder;
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->sharedTempStore = $shared_tempstore;
    $this->mailManager = $mail_manager;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('entity.form_builder'),
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('tempstore.shared'),
      $container->get('plugin.manager.mail'),
      $container->get('language_manager')
    );
  }

  /**
   * Gets the managed user edit form.
   *
   * The user account being edited must have the Managed User role, and the
   * user account performing the edit must have the User Manager role and must
   * also be the User Manager for the account being edited.
   */
  public function getManagedUserEditForm($manager_uid, $user_uid) {
    $current_uid = $this->currentUser->id();
    if ($current_uid === $manager_uid) {
      $user_storage = $this->entityTypeManager->getStorage('user');
      $dpxu_manager = $user_storage->load($manager_uid);
      if ($this->userIsManagerForUser($manager_uid, $user_uid) || $dpxu_manager->hasPermission('administer users')) {
        $dpxu_user = $user_storage->load($user_uid);
        if (is_object($dpxu_user)) {
          return $this->entityFormBuilder->getForm($dpxu_user);
        }
      }
    }
    // If we cannot verify the access, redirect to current user's account page.
    $url = Url::fromRoute('entity.user.canonical', ['user' => $current_uid]);
    return new RedirectResponse($url->toString());
  }

  /**
   * Verify that the manager user is the manager for the user account.
   *
   * @param int $manager_uid
   *   The User Manager id.
   * @param int $user_uid
   *   The Managed User id.
   *
   * @return bool
   *   True or false
   */
  private function userIsManagerForUser($manager_uid, $user_uid) {
    if ($this->userIsManager($manager_uid) && $this->userIsManagedUser($user_uid)) {
      $userManagerId = $this->getUserManagerId($user_uid);
      if ($manager_uid === $userManagerId) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Verify that the user has the User Manager role.
   */
  public function userIsManager($manager_uid) {
    return $this->userHasRole($manager_uid, 'dpxu_manager');
  }

  /**
   * This is just a reusable wrapper for $account->hasRole.
   */
  private function userHasRole($uid, $string) {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $account = $user_storage->load($uid);
    return $account->hasRole($string);
  }

  /**
   * Verify that the user has the Managed User role.
   */
  public function userIsManagedUser($user_uid) {
    return $this->userHasRole($user_uid, 'dpxu_managed');
  }

  /**
   * Get the UID of the User Manager account for this Managed User.
   */
  private function getUserManagerId($user_uid) {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $account = $user_storage->load($user_uid);
    if ($account->hasField('field_dpxu_manager_uid')) {
      $dpxu_manager = $account->get('field_dpxu_manager_uid')->entity;
      if (is_object($dpxu_manager) && $this->userCanCreateManagedUserPermissions($dpxu_manager->id())) {
        return $dpxu_manager->id();
      }
    }
    return NULL;
  }

  /**
   * Verify this user account has a valid non-placeholder email address.
   */
  public function userHasOwnEmailAddress($user_uid) {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $account = $user_storage->load($user_uid);
    if ($account->hasField('mail')) {
      return $this->emailAddressIsValid($account->getEmail());
    }
    return FALSE;
  }

  /**
   * Validate the provided email address.
   */
  public function emailAddressIsValid($email_address) {
    $email_address = trim($email_address);
    if (!empty($email_address) && !str_ends_with($email_address, '.invalid')) {
      return filter_var($email_address, FILTER_VALIDATE_EMAIL);
    }
    return FALSE;
  }

  /**
   * Is account creation enabled in the settings form.
   */
  public function moduleCanCreateAccounts() {
    return $this->configFactory->get('dpxu.settings')->get('dpxu_enabled');
  }

  /**
   * Can the user create a managed user.
   *
   * Returns true if user has dpxu manager role and currently has fewer than
   * the maximum permitted user accounts (configurable in settings page).
   */
  public function userCanCreateManagedUser($uid) {
    if ($this->userCanCreateManagedUserPermissions($uid)) {
      return $this->userCanCreateManagedUserCapacity($uid);
    }
    return FALSE;
  }

  /**
   * Can the user create a managed user - permissions check.
   */
  private function userCanCreateManagedUserPermissions($uid) {
    if ($this->userIsManager($uid) || $this->userHasPermission($uid, 'create dpxu users')) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Can the user create a managed user - capacity check.
   */
  private function userCanCreateManagedUserCapacity($uid) {
    $maxUserCount = $this->configFactory->get('dpxu.settings')->get('dpxu_max_users');
    $currentUserCount = $this->getManagedUserCount($uid);
    if ($currentUserCount < $maxUserCount) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Check whether the module and specified user can create a managed account.
   */
  public function getCreateManagedUserStatusAndReason($uid) {
    $response = [
      'status' => FALSE,
      'message' => $this->t('Creation of managed user accounts is not enabled.'),
    ];
    if ($this->moduleCanCreateAccounts()) {
      $response['message'] = $this->t('Creation of managed user accounts is not enabled for your account.');
      if ($this->userCanCreateManagedUserPermissions($uid)) {
        $response['message'] = $this->configFactory->get('dpxu.settings')->get('dpxu_max_user_message');
        if ($this->userCanCreateManagedUserCapacity($uid)) {
          $response['status'] = TRUE;
          $response['message'] = '';
        }
      }
    }
    return $response;
  }

  /**
   * This is just a reusable wrapper for $account->hasPermission.
   */
  private function userHasPermission($uid, $string) {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $account = $user_storage->load($uid);
    return $account->hasPermission($string);
  }

  /**
   * Returns number of managed users who are assigned to this user manager.
   */
  public function getManagedUserCount($manager_uid) {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $managed_user_query = $user_storage->getQuery()
      ->accessCheck()
      ->condition('status', 1)
      ->condition('field_dpxu_manager_uid', $manager_uid);
    return $managed_user_query->count()->execute();
  }

  /**
   * Checks whether the user has access to the contact form.
   */
  public function userCanSendContactForm($uid) {
    if ($this->userIsManagedUser($uid) && $this->userHasPermission($uid, 'access dpxu contact form')) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Generates a placeholder email address based on the user id.
   */
  public function generateUserEmailAddress($user_uid) {
    return "dpxu_managed_user_" . $user_uid . "@no-mail.invalid";
  }

  /**
   * Returns a list of active user managers.
   */
  public function getManagerList() {
    $list = [];
    $user_storage = $this->entityTypeManager->getStorage('user');
    $dpxu_managers = $user_storage->loadByProperties(['roles' => 'dpxu_manager', 'status' => 1]);
    if (!empty($dpxu_managers)) {
      foreach ($dpxu_managers as $dpxu_manager) {
        $uid = $dpxu_manager->id();
        $list[$uid] = [
          'user_id' => $uid,
          'name' => $dpxu_manager->get('name')->getString(),
          'email' => $dpxu_manager->get('mail')->getString(),
        ];
      }
    }
    return $list;
  }

  /**
   * Function to send a user contact message.
   *
   * @param int $user_uid
   *   The user id.
   * @param array $message
   *   Array: ['key', 'subject', 'body'].
   */
  public function sendUserContactMessage($user_uid, $message) {
    $manager_uid = $this->getUserManagerId($user_uid);
    $mailTo = $this->getUserManagerContactEmail($user_uid, $manager_uid);
    if (!empty($mailTo)) {
      // Set up mail to admin.
      $message['langcode'] = $this->getUserPreferredLangcode($manager_uid);
      if ($message['key'] === 'dpxu_mail_replace') {
        $template = $this->getInterceptTemplate($user_uid, $manager_uid);
        // Clean up intercepted message to remove direct links to user account.
        $message_body = $message['body'];
        if (stripos((string) $message_body, 'http') !== FALSE) {
          $message_body = preg_replace('/https?:\/\/(\S+)/', '', (string) $message_body);
        }
        $message_body = str_replace('[user:one-time-login-url]', '', (string) $message_body);
        // Insert cleaned message into template.
        $message['body'] = str_replace('[dpxu:notification:content]', $message_body, (string) $template);
      }
      else {
        $template = $this->getContactTemplate($user_uid, $manager_uid);
        $message['body'] = str_replace('[dpxu:message:content]', $message['body'], (string) $template);
      }
      return $this->sendUserEmail($mailTo, $message);
    }
    return FALSE;
  }

  /**
   * Gets the email address to send messages to.
   */
  public function getUserManagerContactEmail($user_uid, $manager_uid = '') {
    if (empty($manager_uid)) {
      $manager_uid = $this->getUserManagerId($user_uid);
    }
    if (!empty($manager_uid)) {
      $dpxu_manager_mail = $this->getManagerEmailAddress($manager_uid);
      if (!empty($dpxu_manager_mail)) {
        return $dpxu_manager_mail;
      }
    }
    $config_mail = $this->configFactory->get('dpxu.settings')->get('dpxu_fallback_email');
    if (!empty($config_mail)) {
      return $config_mail;
    }
    return $this->configFactory->get('system.site')->get('mail');
  }

  /**
   * Gets the User Manager's email address.
   */
  private function getManagerEmailAddress($manager_uid) {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $account = $user_storage->load($manager_uid);
    if (is_object($account) && $this->userCanCreateManagedUserPermissions($account->id())) {
      return $account->getEmail();
    }
    return NULL;
  }

  /**
   * Returns the user's preferred language code.
   */
  private function getUserPreferredLangcode($user_uid) {
    if (isset($user_uid)) {
      $user_storage = $this->entityTypeManager->getStorage('user');
      $account = $user_storage->load($user_uid);
      return $account->getPreferredLangcode();
    }
    return $this->languageManager->getDefaultLanguage()->getId();
  }

  /**
   * Gets the template for intercept contact messages.
   */
  private function getInterceptTemplate($user_uid, $manager_uid = '') {
    if (empty($manager_uid)) {
      $manager_uid = $this->getUserManagerId($user_uid);
    }
    $template = $this->configFactory->get('dpxu.settings')
      ->get('dpxu_intercept_template');
    $template = strip_tags((string) $template['value']);
    $custom_template = $this->replaceUserTokens($manager_uid, $user_uid, $template);
    return $custom_template;
  }

  /**
   * Replaces custom tokens.
   */
  private function replaceUserTokens($manager_uid, $user_uid, $content) {
    $dpxu_manager_name = $this->getUserFullName($manager_uid);
    $dpxu_user_name = $this->getUserFullName($user_uid);
    $edit_link = $this->getManagedUserEditLink($user_uid, $manager_uid);

    $known_tokens = [
      '[dpxu:manager:fullname]' => $dpxu_manager_name,
      '[dpxu:user:fullname]' => $dpxu_user_name,
      '[dpxu:user:edit]' => $edit_link,
    ];
    foreach ($known_tokens as $token => $value) {
      $content = str_replace($token, $value, (string) $content);
    }
    return $content;
  }

  /**
   * Gets the user's full name.
   */
  private function getUserFullName($uid) {
    $user_storage = $this->entityTypeManager->getStorage('user');
    $account = $user_storage->load($uid);
    $name_str = '';
    if ($account->hasField('field_dpxu_fullname')) {
      $name_str = $account->get('field_dpxu_fullname')->getString();
    }
    return trim($name_str);
  }

  /**
   * Gets the Managed User edit link.
   */
  public function getManagedUserEditLink($user_uid, $manager_uid = '') {
    if (empty($manager_uid)) {
      $manager_uid = $this->getUserManagerId($user_uid);
    }
    $url = Url::fromRoute('dpxu.user_edit', [
      'manager' => $manager_uid,
      'user' => $user_uid,
    ], ['absolute' => TRUE]);
    return $url->toString();
  }

  /**
   * Gets the contact message template.
   */
  private function getContactTemplate($user_uid, $manager_uid = '') {
    if (empty($manager_uid)) {
      $manager_uid = $this->getUserManagerId($user_uid);
    }
    $template = $this->configFactory->get('dpxu.settings')->get('dpxu_contact_template');
    $template = strip_tags((string) $template['value']);
    $custom_template = $this->replaceUserTokens($manager_uid, $user_uid, $template);
    return $custom_template;
  }

  /**
   * Send a user email.
   *
   * @param string $mail_to
   *   The email address to send the message to.
   * @param array $message
   *   An array containing ['key', 'subject', 'body'].
   */
  private function sendUserEmail($mail_to, $message) {
    // Send mail.
    $module = 'dpxu';
    $key = $message['key'];
    $params['message'] = $message['body'];
    $params['subject'] = $message['subject'];
    $langcode = !empty($message['langcode']) ? $message['langcode'] : LanguageInterface::LANGCODE_NOT_SPECIFIED;
    $send = TRUE;
    $result = $this->mailManager->mail($module, $key, $mail_to, $langcode, $params, NULL, $send);
    return $result['result'];
  }

  /**
   * Save a contact reason to the tempstore.
   *
   * Functions to temporarily store/retrieve data about contact form so we can
   * flag to a manager that an email has been sent to them from their user. We
   * are using the shared tempstore as we need to share the data between two
   * users. This service stores one flag per managed user; dpxu managers are
   * supposed to check their email inbox for the full message(s).
   */
  public function saveContactReasonToTempStore($user_uid, $reason) {
    $tempstore = $this->sharedTempStore->get('dpxu');
    $key = "user_" . $user_uid . "_dpxu_message";
    $flag_reason = '';
    $flag_reason = match ($reason) {
      'dpxu_enquiry' => 'General enquiry',
      'dpxu_update' => 'User requested edit',
      'dpxu_issue' => 'User reported issue',
      'password_reset' => 'User requested new password',
      default => 'Not specified',
    };
    $tempstore->set($key, $flag_reason);
  }

  /**
   * Read the user's latest contact reason from the tempstore.
   */
  public function readContactReasonFromTempStore($user_uid) {
    $tempstore = $this->sharedTempStore->get('dpxu');
    $key = "user_" . $user_uid . "_dpxu_message";
    $reason = $tempstore->get($key);
    return $reason;
  }

  /**
   * Delete a user's last contact reason from the tempstore.
   */
  public function wipeContactReasonFromTempStore($user_uid) {
    $tempstore = $this->sharedTempStore->get('dpxu');
    $key = "user_" . $user_uid . "_dpxu_message";
    $tempstore->delete($key);
  }

}
